package java.util.stream;

import java.util.Iterator;
import java.util.function.LongSupplier;
import java.util.Objects;
import java.util.Arrays;
import java.util.Spliterators;
import java.util.Spliterator;
import java.util.PrimitiveIterator;
import java.util.LongSummaryStatistics;
import java.util.OptionalDouble;
import java.util.function.BiConsumer;
import java.util.function.ObjLongConsumer;
import java.util.function.Supplier;
import java.util.OptionalLong;
import java.util.function.LongBinaryOperator;
import java.util.function.LongConsumer;
import java.util.function.LongToDoubleFunction;
import java.util.function.LongToIntFunction;
import java.util.function.LongFunction;
import java.util.function.LongUnaryOperator;
import java.util.function.LongPredicate;

public interface LongStream extends BaseStream<Long, LongStream>
{
    public abstract LongStream filter(LongPredicate p0);
    
    public abstract LongStream map(LongUnaryOperator p0);
    
    public abstract <U> Stream<U> mapToObj(LongFunction<? extends U> p0);
    
    public abstract IntStream mapToInt(LongToIntFunction p0);
    
    public abstract DoubleStream mapToDouble(LongToDoubleFunction p0);
    
    public abstract LongStream flatMap(LongFunction<? extends LongStream> p0);
    
    public abstract LongStream distinct();
    
    public abstract LongStream sorted();
    
    public abstract LongStream peek(LongConsumer p0);
    
    public abstract LongStream limit(long p0);
    
    public abstract LongStream skip(long p0);
    
    public abstract void forEach(LongConsumer p0);
    
    public abstract void forEachOrdered(LongConsumer p0);
    
    public abstract long[] toArray();
    
    public abstract long reduce(long p0, LongBinaryOperator p1);
    
    public abstract OptionalLong reduce(LongBinaryOperator p0);
    
    public abstract <R> R collect(Supplier<R> p0, ObjLongConsumer<R> p1, BiConsumer<R, R> p2);
    
    public abstract long sum();
    
    public abstract OptionalLong min();
    
    public abstract OptionalLong max();
    
    public abstract long count();
    
    public abstract OptionalDouble average();
    
    public abstract LongSummaryStatistics summaryStatistics();
    
    public abstract boolean anyMatch(LongPredicate p0);
    
    public abstract boolean allMatch(LongPredicate p0);
    
    public abstract boolean noneMatch(LongPredicate p0);
    
    public abstract OptionalLong findFirst();
    
    public abstract OptionalLong findAny();
    
    public abstract DoubleStream asDoubleStream();
    
    public abstract Stream<Long> boxed();
    
    public abstract LongStream sequential();
    
    public abstract LongStream parallel();
    
    public abstract PrimitiveIterator.OfLong iterator();
    
    public abstract Spliterator.OfLong spliterator();
    
    public static LongStream.Builder builder() {
        return new Streams.LongStreamBuilderImpl();
    }
    
    public static LongStream empty() {
        return StreamSupport.longStream(Spliterators.emptyLongSpliterator(), false);
    }
    
    public static LongStream of(long n) {
        return StreamSupport.longStream(new Streams.LongStreamBuilderImpl(n), false);
    }
    
    public static LongStream of(long... array) {
        return Arrays.stream(array);
    }
    
    public static LongStream iterate(final long n, final LongUnaryOperator obj) {
        Objects.requireNonNull(obj);
        return StreamSupport.longStream(Spliterators.spliteratorUnknownSize(new PrimitiveIterator.OfLong() {
            long t = n;
            
            @Override
            public boolean hasNext() {
                return true;
            }
            
            @Override
            public long nextLong() {
                long t = this.t;
                this.t = obj.applyAsLong(this.t);
                return t;
            }
        }, 1296), false);
    }
    
    public static LongStream generate(LongSupplier obj) {
        Objects.requireNonNull(obj);
        return StreamSupport.longStream(new StreamSpliterators.InfiniteSupplyingSpliterator.OfLong(Long.MAX_VALUE, obj), false);
    }
    
    public static LongStream range(long n, long n2) {
        if (n >= n2) {
            return empty();
        }
        if (n2 - n < 0L) {
            long n3 = n + Long.divideUnsigned(n2 - n, 2L) + 1L;
            return concat(range(n, n3), range(n3, n2));
        }
        return StreamSupport.longStream(new Streams.RangeLongSpliterator(n, n2, false), false);
    }
    
    public static LongStream rangeClosed(long n, long n2) {
        if (n > n2) {
            return empty();
        }
        if (n2 - n + 1L <= 0L) {
            long n3 = n + Long.divideUnsigned(n2 - n, 2L) + 1L;
            return concat(range(n, n3), rangeClosed(n3, n2));
        }
        return StreamSupport.longStream(new Streams.RangeLongSpliterator(n, n2, true), false);
    }
    
    public static LongStream concat(LongStream obj, LongStream obj2) {
        Objects.requireNonNull(obj);
        Objects.requireNonNull(obj2);
        return (LongStream)StreamSupport.longStream(new Streams.ConcatSpliterator.OfLong(obj.spliterator(), obj2.spliterator()), obj.isParallel() || obj2.isParallel()).onClose(Streams.composedClose(obj, obj2));
    }
    
    public interface Builder extends LongConsumer
    {
        public abstract void accept(long p0);
        
        default public LongStream.Builder add(long n) {
            this.accept(n);
            return this;
        }
        
        public abstract LongStream build();
    }
}
